define([
  'globals', 'jQuery', 'marionette', 'main/views/header', 'main/views/menu', 'main/views/overlays',
  'main/router', 'penalty/router', 'static/router', 'payment/router', 'refuel/router',
  'request/router', 'search/router', 'delegation/router', 'diary/router'
], function (
  G, $, Marionette, HeaderView, MenuView, OverlaysView,
  MainRouter, PenaltyRouter, StaticRouter, PaymentRouter, RefuelRouter,
  RequestRouter, SearchRouter, DelegationRouter, DiaryRouter
) {
  'use strict';

  var application = new Marionette.Application();

  application.addRegions({
    'header': 'body > header',
    'main': 'body > main',
    'menu': 'body > aside',
    'overlays': '#overlays'
  });

  // Initialize global AJAX configuration
  application.addInitializer(function () {
    $.ajaxSetup({
      headers: {
        'X-Referrer-Id': 'mobile.pomoto.pl'
      }
    });

    $(document).ajaxError(function (event, jqxhr, settings, exception) {
      switch (jqxhr.status) {
        case 401:
        case 403:
        case 500:
          G.identity.clearUserToken();
          G.identity.clearVehicleToken();
          G.identity.verify();
          G.navigate('auth/login');
          break;
      }
    });

    $(document).ajaxStart(function ()  {
      G.vent.trigger('spinner:show');
    });

    $(document).ajaxStop(function () {
      G.vent.trigger('spinner:hide');
    });
  });

  // Initialize routers
  application.addInitializer(function () {
    G.routers.main = new MainRouter();

    if (G.modules.penalty)    G.routers.penalty    = new PenaltyRouter();
    if (G.modules.static)     G.routers.static     = new StaticRouter();
    if (G.modules.payment)    G.routers.payment    = new PaymentRouter();
    if (G.modules.refuel)     G.routers.refuel     = new RefuelRouter();
    if (G.modules.request)    G.routers.request    = new RequestRouter();
    if (G.modules.search)     G.routers.search     = new SearchRouter();
    if (G.modules.delegation) G.routers.delegation = new DelegationRouter();
    if (G.modules.diary)      G.routers.diary      = new DiaryRouter();
  });

  // Advertisments
  application.addInitializer(function () {
    if (G.modules.ads) {
      document.body.classList.remove('no-ads');
    }
  });

  // Initialize sub-views
  application.addInitializer(function () {
    this.header.show(new HeaderView());
    this.menu.show(new MenuView({model: G.identity}));
    this.overlays.show(new OverlaysView());
  });

  // Initialize hardware events
  application.addInitializer(function () {
    var event;

    // menu button
    document.addEventListener('menubutton', function () {
      G.vent.trigger('menu:toggle');
    }, false);

    // back button
    document.addEventListener('backbutton', function () {
      var links = $(document).find('.back');

      if (links.length) {
        window.location.hash = links.get(0).getAttribute('href');
      } else {
        history.back();
      }
    }, false);

    // network state - offline
    document.addEventListener('offline', function () {
      document.body.classList.remove('is-online');
      document.body.classList.add('is-offline');
      G.offline = true;
    });

    // network state - online
    document.addEventListener('online', function () {
      if (G.identity.get('is_verified') && G.identity.get('is_logged')) {
        document.body.classList.remove('is-offline');
        document.body.classList.add('is-online');
        G.offline = false;
      }
    });

    // network checker
    setInterval(function () {

      // detect start of offline mode
      if ((navigator.connection.type == Connection.NONE) && !G.offline) {
        event = document.createEvent('Events');
        event.initEvent('offline');
        document.dispatchEvent(event);
      }

      // detect start of online mode
      if ((navigator.connection.type !== Connection.NONE) && G.offline) {
        event = document.createEvent('Events');
        event.initEvent('online');
        document.dispatchEvent(event);
      }
    }, 1000);

    // initial conditional offline state
    if (navigator.connection.type == Connection.NONE) {
      event = document.createEvent('Events');
      event.initEvent('offline');
      document.dispatchEvent(event);
    }
  });

  application.addInitializer(function () {
    if (navigator.userAgent.toLowerCase().indexOf('android') > -1) {
      document.body.classList.add('android');
    } else {
      document.body.classList.add('ios');
    }
  });

  // Initialize the Identity model
  application.addInitializer(function () {
    if (!G.offline) {
      G.identity.verify();
    }
  });

  // Initialize GPS recorder
  application.addInitializer(function () {
    G.identity.on('change', function (identity) {
      if (identity.get('is_verified') && identity.get('is_logged')) {
        G.recorder.initialize();
        G.vent.trigger('record:init');
      }
    });
  });

  application.on('initialize:after', function () {
    document.body.classList.add('ready');
    window.console.info('Application started.');
  });

  return application;
});
