define(['underscore', 'module'], function (_, module) {
  'use strict';

  var Recorder = {

    // true if recorder is started
    enabled: false,

    // watch ID (resource)
    watch: false,

    // item data
    model: {},

    // trace buffer
    buffer: [],

    // flush limit
    limit: 60,

    // false only during flushing
    flushable: true,

    // true if segment is finished
    newSegment: true,

    initialize: function () {
      _.bindAll(this,
        'init', 'start', 'pause', 'resume', 'stop', 'update',
        'geolocationSuccess', 'geolocationError'
      );

      G.vent.on('record:init'  , this.init  );
      G.vent.on('record:start' , this.start );
      G.vent.on('record:pause' , this.pause );
      G.vent.on('record:resume', this.resume);
      G.vent.on('record:stop'  , this.stop  );
      G.vent.on('record:update', this.update);
    },

    init: function () {
      var self = this;

      console.log('init');

      $.get(module.config().item, function(data, status, xhr) {
          if (!data || !data.id) return;

          console.info('init', data);

          self.model = data;
          G.vent.trigger('record:start');
          G.vent.trigger('record:distance', data.distance);
        });

    },

    start: function () {
      if (this.enabled) return;
      this.enabled = true;

      console.log('start');

      G.vent.trigger('record:status', 'in-progress');

      this.watch = navigator.geolocation.watchPosition(
        this.geolocationSuccess,
        this.geolocationError,
        { enableHighAccuracy: true }
      );
    },

    pause: function () {
      if (!this.enabled) return;

      console.log('pause');

      if (this.watch) {
        navigator.geolocation.clearWatch(this.watch);
        this.watch = false;
      }
    },

    resume: function () {
      if (!this.enabled) return;

      console.log('resume');

      this.newSegment = true;
      G.vent.trigger('record:status', 'in-progress');

      this.watch = navigator.geolocation.watchPosition(
        this.geolocationSuccess,
        this.geolocationError,
        { enableHighAccuracy: true }
      );
    },

    stop: function () {
      if (!this.enabled) return;

      this.enabled = false;

      console.log('stop');

      if (this.watch) {
        navigator.geolocation.clearWatch(this.watch);
        this.watch = false;
      }

      var self = this;
      $.ajax({
        type: 'DELETE',
        url: module.config().close + this.model.id,

        success: function(data, status, xhr) {
          console.info('stop', data);
        }
      });

      G.navigate('delegation/items/' + self.model.fk_fleet_delegations_id);
    },

    update: function () {
      console.info('update', this.model.distance);
      G.vent.trigger('record:distance', this.model.distance);
    },

    geolocationSuccess: function (location) {
      var self = this;
      console.info('GPS', location);

      G.vent.trigger('record:status', 'success');

      this.buffer.push({
        alt: location.coords.altitude || 0,
        lat: location.coords.latitude || 0,
        lng: location.coords.longitude || 0,
        spd: location.coords.speed || 0
      });

      if (this.newSegment) {
        this.buffer[this.buffer.length-1].seg = true;
        this.newSegment = false;
      }

      if (this.flushable && (this.buffer.length >= this.limit)) {
        console.log('flushable');

        $.ajax({
          type: 'POST',
          url: module.config().flush,
          data: JSON.stringify({
            id: this.model.id,
            data: this.buffer
          }),

          success: function(data, status, xhr) {
            console.info('flush', 'success', data);
            self.buffer = [];
            self.flushable = true;
            G.vent.trigger('record:distance', data.distance);
          },
          error: function (xhr, status, error) {
            console.error('flush', 'error', status, error);
            self.flushable = true;
          }
        });

        this.flushable = false;
      }
    },

    geolocationError: function (error) {
      console.error('GPS', error);
      G.vent.trigger('record:status', 'error');
    }
  };

  return Recorder;
});
