define(['marionette'], function (Marionette) {
  'use strict';

  var DelegationRouter = Marionette.AppRouter.extend({

    routes: {
      'delegation': 'delegationList',
      'delegation/view/:id': 'viewDelegation',
      'delegation/add': 'addDelegation',
      'delegation/edit/:id': 'editDelegation',
      'delegation/items/:id': 'delegationItems',
      'delegation/item/:id': 'delegationItem',
      'delegation/item/add/:id': 'addItem',
      'delegation/item/track/:id': 'trackItem',
      'delegation/record': 'record'
    },

    delegationList: function () {
      this._ifLogged(function () {
        require(['application', 'delegation/views/list', 'delegation/collections/delegation'], function (application, DelegationListView, DelegationCollection) {
          var collection = new DelegationCollection(),
              view = new DelegationListView({collection: collection});

          application.main.show(view);
          collection.fetch();
        });
      });
    },

    viewDelegation: function (id) {
      this._ifLogged(function () {
        require(['application', 'delegation/views/view', 'delegation/models/delegation'], function (application, DelegationViewView, DelegationModel) {
          var model = new DelegationModel({id: id}),
              view = new DelegationViewView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    addDelegation: function () {
      this._ifLogged(function () {
        require(['application', 'delegation/views/add', 'delegation/models/delegation'], function (application, DelegationAddView, DelegationModel) {
          var model = new DelegationModel(),
              view = new DelegationAddView({model: model});

          application.main.show(view);
        });
      });
    },

    editDelegation: function (id) {
      this._ifLogged(function () {
        require(['application', 'delegation/views/edit', 'delegation/models/delegation'], function (application, DelegationEditView, DelegationModel) {
          var model = new DelegationModel({id: id}),
              view = new DelegationEditView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    delegationItems: function (id) {
      this._ifLogged(function () {
        require(['application', 'delegation/views/items-list', 'delegation/collections/item'], function (application, DelegationItemsListView, ItemCollection) {
          var collection = new ItemCollection(),
              view = new DelegationItemsListView({collection: collection});

          collection.id = id;
          application.main.show(view);
          collection.fetch();
        });
      });
    },

    addItem: function (id) {
      this._ifLogged(function () {
        require(['application', 'delegation/views/item-add', 'delegation/models/item'], function (application, ItemAddView, ItemModel) {
          var model = new ItemModel({fk_fleet_delegations_id: id}),
              view = new ItemAddView({model: model});

          application.main.show(view);
        });
      });
    },

    trackItem: function (id) {
      this._ifLogged(function () {
        require(['application', 'delegation/views/item-track', 'delegation/models/item'], function (application, ItemTrackView, ItemModel) {
          var model = new ItemModel({fk_fleet_delegations_id: id}),
              view = new ItemTrackView({model: model});

          application.main.show(view);
        });
      });
    },

    record: function () {
      this._ifLogged(function () {
        require(['application', 'delegation/views/record'], function (application, RecordView) {
          var view = new RecordView();
          application.main.show(view);
        });
      });
    },

    // Shortcut to main routers _ifLogged method
    _ifLogged: function (callback) {
      G.routers.main._ifLogged.call(this, callback);
    }
  });

  return DelegationRouter;
});
