define(
  ['marionette', 'hb!delegation/templates/add', 'module'],
  function (Marionette, delegationAddTemplate, module) {
  'use strict';

  var DelegationAdd = Marionette.ItemView.extend({
    template: delegationAddTemplate,

    events: {
      'change #fk_fleet_area_id': 'areaChange',
      'focus [type=date]': 'setDate',
      'submit form': 'addDelegation'
    },

    modelEvents: {
      'change': 'splitDates render areaChange' // areaChange - render of users
    },

    initialize: function () {
      var self = this;

      G.vent.trigger('header-title:change', 'Delegacje');

      _.bindAll(this, 'areaChange');

      $.get(module.config().fleetData, function (data, status, xhr) {
        self.model.set('fleetData', data);
      });
    },

    splitDates: function (e) {
      var date_start_travel = this.model.get('date_start_travel'),
          date_end_travel = this.model.get('date_end_travel');

      if (date_start_travel && date_start_travel.length > 10) {
        this.model.set({
          dt_start: date_start_travel.substr(0, 10),
          hr_start: parseInt(date_start_travel.substr(11, 2), 10),
          mn_start: date_start_travel.substr(14, 2)
        });
      }

      if (date_end_travel && date_end_travel.length > 10) {
        this.model.set({
          dt_end: date_end_travel.substr(0, 10),
          hr_end: parseInt(date_end_travel.substr(11, 2), 10),
          mn_end: date_end_travel.substr(14, 2)
        });
      }
    },

    areaChange: function (e) {
      var id = parseInt($('#fk_fleet_area_id').val(), 10),
          users, option;

      $('[name=fk_user_user_id]').html('');
      if (isNaN(id)) return;

      if (this.model.get('fleetData') && this.model.get('fleetData')[id]) {
        users = this.model.get('fleetData')[id].users;

        if (Object.keys(users).length > 1) {
          $('[name=fk_user_user_id]').html('<option></option>');
        } else {
          $('[name=fk_user_user_id]').html('');
        }

        for (id in users) {
          option = document.createElement('option');
          option.innerHTML = users[id].name;
          option.setAttribute('value', id);

          if (id == this.model.get('fk_user_user_id')) {
            option.setAttribute('selected', 'selected');
          }

          $('[name=fk_user_user_id]').append(option);
        }
      }
    },

    addDelegation: function (e) {
      var self = this,
          date_start, date_end;

      date_start = this.$el.find('[name=time_start_travel]').val();
      if (date_start.length == 4) date_start = "0" + date_start;
      date_start = this.$el.find('[name=date_start_travel]').val() + ' ' + date_start;

      date_end = this.$el.find('[name=time_end_travel]').val();
      if (date_end.length == 4) date_end = "0" + date_end;
      date_end = this.$el.find('[name=date_end_travel]').val() + ' ' + date_end;

      this.model.save({
        working_name: this.$el.find('[name=working_name]').val(),
        fk_fleet_area_id: this.$el.find('[name=fk_fleet_area_id]').val(),
        fk_user_user_id: this.$el.find('[name=fk_user_user_id]').val(),

        target_place: this.$el.find('[name=target_place]').val(),
        source_place: this.$el.find('[name=source_place]').val(),
        date_start_travel: date_start,
        date_end_travel: date_end,

        travel_lump_sum: this.$el.find('[name=travel_lump_sum]').val(),
        travel_cost: this.$el.find('[name=travel_cost]').val(),
        accommodation_bill: this.$el.find('[name=accommodation_bill]').val(),
        accommodation_lump_sum: this.$el.find('[name=accommodation_lump_sum]').val(),
        expenses: this.$el.find('[name=expenses]').val(),
        installment: this.$el.find('[name=installment]').val()
      }, {
        success: function (model, resp, options) {
          G.navigate('delegation');
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'working_name')
                .showErrorMessage(resp, 'fk_fleet_area_id')
                .showErrorMessage(resp, 'fk_user_user_id')

                .showErrorMessage(resp, 'target_place')
                .showErrorMessage(resp, 'source_place')
                .showErrorMessage(resp, 'start_travel')
                .showErrorMessage(resp, 'end_travel')

                .showErrorMessage(resp, 'travel_lump_sum')
                .showErrorMessage(resp, 'travel_cost')
                .showErrorMessage(resp, 'accommodation_bill')
                .showErrorMessage(resp, 'accommodation_lump_sum')
                .showErrorMessage(resp, 'expenses')
                .showErrorMessage(resp, 'installment')
              ;
              break;
          }
        }
      });

      return false;
    },

    setDate: function (e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + '], .' + name).next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return DelegationAdd;
});
