define(
  ['marionette', 'hb!delegation/templates/item-track', 'module'],
  function (Marionette, itemTrackTemplate, module) {
  'use strict';

  var ItemTrack = Marionette.ItemView.extend({
    template: itemTrackTemplate,

    events: {
      'focus [type=date]': 'setDate',
      'submit form': 'addItem'
    },

    modelEvents: {
      'change': 'splitDates render'
    },

    initialize: function () {
      var self = this;

      G.vent.trigger('header-title:change', 'Delegacje');

      $.get(module.config().vehicleData, function (data, status, xhr) {
        self.model.set('vehicleData', data);
      });
    },

    splitDates: function (e) {
      var travel_start = this.model.get('travel_start'),
          travel_end = this.model.get('travel_end');

      if (travel_start && travel_start.length > 10) {
        this.model.set({
          dt_start: travel_start.substr(0, 10),
          hr_start: parseInt(travel_start.substr(11, 2), 10),
          mn_start: travel_start.substr(14, 2)
        });
      }

      if (travel_end && travel_end.length > 10) {
        this.model.set({
          dt_end: travel_end.substr(0, 10),
          hr_end: parseInt(travel_end.substr(11, 2), 10),
          mn_end: travel_end.substr(14, 2)
        });
      }
    },

    addItem: function (e) {
      var self = this,
          travel_start, travel_end;

      travel_start = this.$el.find('[name=travel_start_time]').val();
      if (travel_start.length == 4) travel_start = "0" + travel_start;
      travel_start = this.$el.find('[name=travel_start_date]').val() + ' ' + travel_start;

      travel_end = this.$el.find('[name=travel_end_time]').val();
      if (travel_end.length == 4) travel_end = "0" + travel_end;
      travel_end = this.$el.find('[name=travel_end_date]').val() + ' ' + travel_end;

      this.model.save({
        fk_fleet_delegations_id: this.$el.find('[name=fk_fleet_delegations_id]').val(),
        fk_vehicle_vehicle_id: this.$el.find('[name=fk_vehicle_vehicle_id]').val(),

        travel_start: travel_start,
        travel_end: travel_end,

        target_start: this.$el.find('[name=target_start]').val(),
        target_end: this.$el.find('[name=target_end]').val(),

        travel_length: 0
      }, {
        success: function (model, resp, options) {
          G.vent.trigger('record:init');
          G.navigate('delegation/record');
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'fk_fleet_delegations_id')

                .showErrorMessage(resp, 'travel_start')
                .showErrorMessage(resp, 'travel_end')

                .showErrorMessage(resp, 'target_start')
                .showErrorMessage(resp, 'target_end')
                .showErrorMessage(resp, 'travel_length')
              ;
              break;
          }
        }
      });

      return false;
    },

    setDate: function (e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + '], .' + name).next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return ItemTrack;
});
