define(['marionette'], function (Marionette) {
  'use strict';

  var DiaryRouter = Marionette.AppRouter.extend({

    routes: {
      'diary': 'diaryList',
      'diary/add': 'addDiary',
      'diary/view/:id': 'viewDiary',
      'diary/edit/:id': 'editDiary',
      'diary/parts/:id': 'viewParts',
      'diary/parts/:id/add': 'addPart',
      'diary/parts/:diary/edit/:part': 'editPart',
    },

    diaryList: function () {
      this._ifLogged(function () {
        require(['application', 'diary/views/list', 'diary/collections/diary'], function (application, DiaryListView, DiaryCollection) {
          var collection = new DiaryCollection(),
              view = new DiaryListView({collection: collection});

          application.main.show(view);
          collection.fetch();
        });
      });
    },

    addDiary: function () {
      this._ifLogged(function () {
        require(['application', 'diary/views/add', 'diary/models/diary'], function (application, DiaryAddView, DiaryModel) {
          var model = new DiaryModel(),
              view = new DiaryAddView({model: model});

          application.main.show(view);
        });
      });
    },

    viewDiary: function (id) {
      this._ifLogged(function () {
        require(['application', 'diary/views/view', 'diary/models/diary'], function (application, DiaryViewView, DiaryModel) {
          var model = new DiaryModel({id: id}),
              view = new DiaryViewView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    editDiary: function (id) {
      this._ifLogged(function () {
        require(['application', 'diary/views/edit', 'diary/models/diary'], function (application, DiaryAddView, DiaryModel) {
          var model = new DiaryModel({id: id}),
              view = new DiaryAddView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    viewParts: function (id) {
      this._ifLogged(function () {
        require(['application', 'diary/views/parts', 'diary/collections/part'], function (application, PartsListView, PartCollection) {
          var collection = new PartCollection({id: id}),
              view = new PartsListView({collection: collection});

          collection.id = id;
          application.main.show(view);
          collection.fetch();
        });
      });
    },

    addPart: function (id) {
      this._ifLogged(function () {
        require(['application', 'diary/views/part-add', 'diary/models/part'], function (application, PartAddView, PartModel) {
          var model = new PartModel({fk_diary_diary_id: id}),
              view = new PartAddView({model: model});

          application.main.show(view);
        });
      });
    },

    editPart: function (diary, part) {
      this._ifLogged(function () {
        require(['application', 'diary/views/part-edit', 'diary/models/part'], function (application, PartEditView, PartModel) {
          var model = new PartModel({
                id: part,
                fk_diary_diary_id: diary
              }),
              view = new PartEditView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    // Shortcut to main routers _ifLogged method
    _ifLogged: function (callback) {
      G.routers.main._ifLogged.call(this, callback);
    }
  });

  return DiaryRouter;
});
