define(
  ['marionette', 'diary/models/diary', 'module', 'hb!diary/templates/add', 'moment'],
  function (Marionette, DiaryModel, module, diaryAddTemplate, moment) {
  'use strict';

  var DiaryAdd = Marionette.ItemView.extend({
    template: diaryAddTemplate,

    events: {
      'submit form': 'addDiary',
      'focus [type=date]': 'setDate'
    },

    modelEvents: {
      'change': 'splitDates render'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Dziennik zdarzeń');

      var i, self = this;
      $.get(module.config().workshopTypes, function (data, status, xhr) {
        self.model.set('workshopType', data);
      });
    },

    splitDates: function (e) {
      var date_start = this.model.get('date_start'),
          date_end = this.model.get('date_end');

      if (date_start && date_start.length > 10) {
        this.model.set({
          dt_start: date_start.substr(0, 10),
          hr_start: parseInt(date_start.substr(11, 2), 10),
          mn_start: date_start.substr(14, 2)
        });
      }

      if (date_end && date_end.length > 10) {
        this.model.set({
          dt_end: date_end.substr(0, 10),
          hr_end: parseInt(date_end.substr(11, 2), 10),
          mn_end: date_end.substr(14, 2)
        });
      }
    },

    addDiary: function (e) {
      var self = this,
          date_start, date_end;

      date_start = this.$el.find('[name=start_time]').val();
      if (date_start.length == 4) date_start = "0" + date_start;
      date_start = this.$el.find('[name=start_date]').val() + ' ' + date_start;

      date_end = this.$el.find('[name=end_time]').val();
      if (date_end.length == 4) date_end = "0" + date_end;
      date_end = this.$el.find('[name=end_date]').val() + ' ' + date_end;

      this.model.save({
        work_title: this.$el.find('[name=work_title]').val(),
        fk_workshop_type_id: this.$el.find('[name=fk_workshop_type_id]').val(),
        price: this.$el.find('[name=price]').val(),

        date_start: date_start,
        date_end: date_end,

        workshop_name: this.$el.find('[name=workshop_name]').val(),
        workshop_city: this.$el.find('[name=workshop_city]').val(),
        workshop_address: this.$el.find('[name=workshop_address]').val(),
        workshop_postal_code: this.$el.find('[name=workshop_postal_code]').val(),

        repeat: this.$el.find('[name=repeat]').val(),
        remind: this.$el.find('[name=remind]').val(),
        remind_after: this.$el.find('[name=remind_after]').val(),

        content_description: this.$el.find('[name=content_description]').val()
      }, {
        success: function (model, resp, options) {
          G.navigate('diary');
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'work_title')
                .showErrorMessage(resp, 'fk_workshop_type_id')
                .showErrorMessage(resp, 'price')

                .showErrorMessage(resp, 'date_start')
                .showErrorMessage(resp, 'date_end')

                .showErrorMessage(resp, 'workshop_name')
                .showErrorMessage(resp, 'workshop_city')
                .showErrorMessage(resp, 'workshop_address')
                .showErrorMessage(resp, 'workshop_postal_code')

                .showErrorMessage(resp, 'repeat')
                .showErrorMessage(resp, 'remind')
                .showErrorMessage(resp, 'remind_after')
                .showErrorMessage(resp, 'content_description')
              ;
              break;
          }
        }
      });

      return false;
    },

    setDate: function (e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + '], .' + name).next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return DiaryAdd;
});
