define(
  ['marionette', 'module', 'hb!diary/templates/part-edit', 'moment'],
  function (Marionette, module, partEditTemplate, moment) {
  'use strict';

  var PartEdit = Marionette.ItemView.extend({
    template: partEditTemplate,

    events: {
      'click .delete': 'deletePart',
      'submit form': 'savePart',
      'change [name=unit]': 'changeUnit'
    },

    modelEvents: {
      'change': 'render'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Dziennik zdarzeń');

      var i, self = this;
      $.get(module.config().units, function (data, status, xhr) {
        self.model.set('units', data);
      });
    },

    deletePart: function () {
      var self = this;

      if (!confirm('Czy jesteś pewien?')) return false;

      this.model.destroy({
        success: function (model, response) {
          G.navigate('diary/parts/' + self.model.get('fk_diary_diary_id'));
        }
      });
      return false;
    },

    savePart: function (e) {
      var self = this;

      this.model.save({
        title: this.$el.find('[name=title]').val(),
        amount: this.$el.find('[name=amount]').val(),
        unit: this.$el.find('[name=unit]').val(),
        price: this.$el.find('[name=price]').val()
      }, {
        success: function (model, resp, options) {
          G.navigate('diary/parts/' + self.model.get('fk_diary_diary_id'));
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'title')
                .showErrorMessage(resp, 'amount')
                .showErrorMessage(resp, 'unit')
                .showErrorMessage(resp, 'price')
              ;
              break;
          }
        }
      });

      return false;
    },

    changeUnit: function () {
      var i, units = this.model.get('units'),
          unitId = this.$el.find('[name=unit]').val();

      //...this.$el.find('[name=amount]').attr('placeholder', '');
      for (i in units) {
        if (units[i].id == unitId) {
          //...this.$el.find('[name=amount]').attr('placeholder', units[i].name);
        }
      }
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + ']').next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return PartEdit;
});
