define(['jQuery', 'backbone', 'module'], function ($, Backbone, module) {
  'use strict';

  var Identity = Backbone.Model.extend({
    defaults: {
      is_verified: false,
      is_logged: undefined,
      user: undefined
    },

    url: module.config().url,

    initialize: function () {
      this.setUserToken(this.getUserToken());
      this.setVehicleToken(this.getVehicleToken());
    },

    verify: function () {
      this.set('is_verified', false);

      this.fetch({
        success: function (model, data, request) {
          model.set('is_verified', true);
        },
        error: function (model, data, request) {
          //...
        }
      });
    },

    getUserToken: function () {
      if (window.localStorage.getItem('userToken') !== null) {
        return window.localStorage.getItem('userToken');
      } else {
        return '';
      }
    },

    setUserToken: function (token) {
      window.localStorage.setItem('userToken', token);
      $.ajaxSetup({
        headers: {
          'X-User-Token': this.getUserToken()
        }
      });
    },

    clearUserToken: function() {
      this.setUserToken('');
    },

    getVehicleToken: function () {
      if (window.localStorage.getItem('vehicleToken') !== null) {
        return window.localStorage.getItem('vehicleToken');
      } else {
        return '';
      }
    },

    setVehicleToken: function (token) {
      window.localStorage.setItem('vehicleToken', token);
      $.ajaxSetup({
        headers: {
          'X-Vehicle-Token': this.getVehicleToken()
        }
      });
    },

    clearVehicleToken: function() {
      this.setVehicleToken('');
    }
  });

  return Identity;
});
