define(['marionette'], function (Marionette) {
  'use strict';

  var MainRouter = Marionette.AppRouter.extend({

    routes: {
      '': 'showHome',
      'auth/register': 'showRegister',
      'auth/login': 'showLogin',
      'auth/vehicle': 'vehicleList',
      '*other': 'error404'
    },

    showHome: function () {
      this._ifLoggedOrOffline(function () {
        require(['application', 'main/views/home', 'main/models/blank'], function (application, HomeView, BlankModel) {
          var model = new BlankModel(),
              view = new HomeView({model: model});
          application.main.show(view);
        });
      });
    },

    showLogin: function () {
      require(['application', 'main/views/login'], function (application, LoginView) {
        application.main.show(new LoginView());
      });
    },

    showRegister: function () {
      require(['application', 'main/views/register', 'main/models/registration'], function (application, RegisterView, RegistrationModel) {
        var model = new RegistrationModel(),
            view = new RegisterView({model: model});
        application.main.show(view);
      });
    },

    vehicleList: function () {
      require(['application', 'main/views/vehicle-list', 'main/collections/vehicle'], function (application, VehicleList, VehicleCollection) {
        var collection = new VehicleCollection({}),
            view = new VehicleList({collection: collection});

        application.main.show(view);
        collection.fetch();
      });
    },

    error404: function (url) {
      window.console.error('Route "' + url + '" does not exist!');
    },

    /**
     * Execute callback only when user is verified.
     * @param callback code to execute if user is logged
     */
    _ifLogged: function (callback) {
      if (G.identity.get('is_verified')) {
        if (G.identity.get('is_logged')) {
          callback.call(this);
        } else {
          G.navigate('auth/login');
        }
      } else {
        G.identity.once('change:is_verified', function (model, value) {
          this._ifLogged(callback);
        }, this);
      }
    },

    _ifLoggedOrOffline: function (callback) {
      if(G.offline) {
        callback.call(this);
      } else {
        this._ifLogged(callback);
      }
    }
  });

  return MainRouter;
});
