define(['jQuery', 'marionette', 'hb!main/templates/header', 'module'], function ($, Marionette, headerTemplate, module) {
  'use strict';

  var Header = Marionette.ItemView.extend({

    template: headerTemplate,

    events: {
      'click .menu': 'toggleMenu'
    },

    messageInterval: null,
    messageCount: 0,

    initialize: function () {
      var self = this;

      $(document).ajaxComplete(function (event, xhr, settings) {
        var unread = xhr.getResponseHeader('X-Messages-Unread');

        if (/[0-9]+/.test(unread)) {
           unread = (unread === null) ? 0 : parseInt(unread, 10);

          if (unread > 0) {
            self.$el.find('.request').fadeIn();
            if (unread > self.messageCount) {
              navigator.notification.vibrateWithPattern([0, 600, 300, 600]);
            }
          } else {
            self.$el.find('.request').fadeOut();
          }

          self.messageCount = unread;
          clearInterval(self.messageInterval);

          self.messageInterval = setInterval(function () {
            if (!G.offline && G.identity.get('is_verified') && G.identity.get('is_logged')) {
              G.vent.trigger('spinner:disable');
              $.get(module.config().messages.url, function () {
                G.vent.trigger('spinner:enable');
              });
            }
          }, module.config().messages.interval * 1000);
        }
      });

      if (!G.offline && G.identity.get('is_verified') && G.identity.get('is_logged')) {
        $.get(module.config().messages.url);
      }

      this.fetchOfflineData();

      G.vent.on('header-title:change', function (title) {
        this.$el.find('h1').html(title);
      }, this);

      G.vent.on('record:start', function () {
        self.$el.find('.record').fadeIn();
      }, this);

      G.vent.on('record:stop', function () {
        self.$el.find('.record').fadeOut();
      }, this);
    },

    fetchOfflineData: function () {
      var self = this;

      if (!G.offline && G.identity.get('is_verified') && G.identity.get('is_logged')) {
        navigator.geolocation.getCurrentPosition(
          function (location) {
            var query = 'lat=' + location.coords.latitude + '&lng=' + location.coords.longitude;

            if (localStorage['search/local-list/lat'] && localStorage['search/local-list/lng']) {
              query += '&prev_lat=' + localStorage['search/local-list/lat'] + '&prev_lng=' + localStorage['search/local-list/lng'];
            }

            if (!G.offline && G.identity.get('is_logged')) {
              G.vent.trigger('spinner:disable');

              $.get(module.config().offline.url + query, function (data) {
                var jsonData = [];

                try {
                  jsonData = JSON.parse(data);

                  if ((jsonData instanceof Array) && (jsonData.length)) {
                    localStorage['search/local-list/data'] = data;
                    localStorage['search/local-list/lat'] = location.coords.latitude;
                    localStorage['search/local-list/lng'] = location.coords.longitude;
                  }
                } catch (e) {}

                G.vent.trigger('spinner:enable');
              }, 'text');
            }
          },

          function () {},
          { enableHighAccuracy: true }
        );
      }

      setTimeout(function () {
        self.fetchOfflineData();
      }, module.config().offline.interval * 1000);
    },

    toggleMenu: function (e) {
      G.vent.trigger('menu:toggle');
    }
  });

  return Header;
});
