define(['jQuery', 'marionette', 'hb!main/templates/register', 'module'], function ($, Marionette, registerTemplate, module) {
  'use strict';

  var Register = Marionette.ItemView.extend({

    template: registerTemplate,

    events: {
      'change [name=fk_vehicle_type_id], [name=production_year]': 'updateBranch',
      'change [name=fk_vehicle_branch_id]': 'updateModel',
      'focus [type=date]': 'setDate',
      'click  #register-form-goto2': 'showStep2',
      'click  #register-form-goto1': 'showStep1',
      'submit form': 'register'
    },

    initialize: function () {
      var self = this, i, option;

      G.vent.trigger('header-title:change', 'Rejestracja');

      _.bindAll(this, 'updateBranch', 'updateModel');

      $.get(module.config().vehicleType, function (data, status, xhr) {
        $('[name=fk_vehicle_type_id]').html('');

        for (i in data) {
          option = document.createElement('option');
          option.innerHTML = data[i].name;
          option.setAttribute('value', data[i].id);

          if (data[i].id == self.model.get('fk_vehicle_type_id')) {
            option.setAttribute('selected', 'selected');
          }

          $('[name=fk_vehicle_type_id]').append(option);
        }

        self.updateBranch();
      });

      $.get(module.config().vehicleColor, function (data, status, xhr) {
        $('[name=fk_vehicle_color_id]').html('');

        for (i in data) {
          option = document.createElement('option');
          option.innerHTML = data[i].name;
          option.setAttribute('value', data[i].id);

          if (data[i].id == self.model.get('fk_vehicle_color_id')) {
            option.setAttribute('selected', 'selected');
          }

          $('[name=fk_vehicle_color_id]').append(option);
        }
      });
    },

    updateBranch: function (e) {
      var self = this, key, option,
          type = this.$el.find('[name=fk_vehicle_type_id]').val(),
          year = this.$el.find('[name=production_year]').val();

      $.get(module.config().vehicleBranch, {fk_vehicle_type_id: type, year: year}, function (data, status, xhr) {
        $('[name=fk_vehicle_branch_id]').html('');

        for (key in data) {
          option = document.createElement('option');
          option.innerHTML = data[key];
          option.setAttribute('value', key);

          if (key == self.model.get('fk_vehicle_branch_id')) {
            option.setAttribute('selected', 'selected');
          }

          $('[name=fk_vehicle_branch_id]').append(option);
        }

        self.updateModel();
      });
    },

    updateModel: function (e) {
      var self = this, key, option,
          type = this.$el.find('[name=fk_vehicle_type_id]').val(),
          year = this.$el.find('[name=production_year]').val(),
          branch = this.$el.find('[name=fk_vehicle_branch_id]').val();

      $.get(module.config().vehicleModel, {fk_vehicle_type_id: type, year: year, fk_vehicle_branch_id: branch}, function (data, status, xhr) {
        $('[name=fk_vehicle_model_id]').html('');

        for (key in data) {
          option = document.createElement('option');
          option.innerHTML = data[key];
          option.setAttribute('value', key);

          if (key == self.model.get('fk_vehicle_model_id')) {
            option.setAttribute('selected', 'selected');
          }

          $('[name=fk_vehicle_model_id]').append(option);
        }
      });
    },

    setDate: function (e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showStep2: function () {
      this.$el.find('#register-form-step1').hide();
      this.$el.find('#register-form-step2').show();
      return false;
    },

    showStep1: function () {
      this.$el.find('#register-form-step2').hide();
      this.$el.find('#register-form-step1').show();
      return false;
    },

    register: function (e) {
      var self = this,
          password1, password2, errorLabel;

      password1 = this.$el.find('[name=password]').val();
      password2 = this.$el.find('[name=confirm_password]').val();
      errorLabel = this.$el.find('[name=confirm_password]').next('.error');

      if (password1 != password2) {
        errorLabel.text('Niezgodne hasła');
        this.showStep1();
        return false;
      } else {
        errorLabel.text('');
      }

      this.model.save({
        id: 0,

        User_Form_MobileRegisterUser: {
          email: this.$el.find('[name=email]').val(),
          password: this.$el.find('[name=password]').val(),
          process: 1,
        },

        User_Form_MobileRegisterVehicle: {
          working_name: this.$el.find('[name=working_name]').val(),
          fk_vehicle_type_id: this.$el.find('[name=fk_vehicle_type_id]').val(),
          production_year: this.$el.find('[name=production_year]').val(),
          fk_vehicle_branch_id: this.$el.find('[name=fk_vehicle_branch_id]').val(),
          fk_vehicle_model_id: this.$el.find('[name=fk_vehicle_model_id]').val(),
          fk_vehicle_color_id: this.$el.find('[name=fk_vehicle_color_id]').val(),
          registration_number: this.$el.find('[name=registration_number]').val(),
          registration_date: this.$el.find('[name=registration_date]').val(),
        }
      }, {
        success: function (model, resp, options) {
          G.identity.setUserToken(resp['X-User-Token']);
          G.identity.setVehicleToken(resp['X-Vehicle-Token']);
          G.identity.verify();

          G.navigate('');
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'User_Form_MobileRegisterUser', 'email')
                .showErrorMessage(resp, 'User_Form_MobileRegisterUser', 'password')

                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'working_name')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'fk_vehicle_type_id')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'production_year')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'fk_vehicle_branch_id')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'fk_vehicle_model_id')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'fk_vehicle_color_id')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'registration_number')
                .showErrorMessage(resp, 'User_Form_MobileRegisterVehicle', 'registration_date')
              ;
              break;
          }
        }
      });

      return false;
    },

    showErrorMessage: function (resp, group, name) {
      var label = this.$el.find('[name=' + name + '], .' + name).next('.error');

      if (resp.responseJSON && resp.responseJSON[group] && resp.responseJSON[group][name]) {
        label.text(resp.responseJSON[group][name]);

        if (label.parents('#register-form-step1').length) {
          this.showStep1();
        }
      } else {
        label.text('');
      }
      return this;
    }
  });

  return Register;
});
