define(['marionette'], function (Marionette) {
  'use strict';

  var PenaltyRouter = Marionette.AppRouter.extend({

    routes: {
      'penalty': 'penaltyList',
      'penalty/add': 'addPenalty',
      'penalty/view/:id': 'viewPenalty',
      'penalty/edit/:id': 'editPenalty'
    },

    penaltyList: function () {
      this._ifLogged(function () {
        require(['application', 'penalty/views/list', 'penalty/collections/penalty'], function (application, PenaltyListView, PenaltyCollection) {
          var collection = new PenaltyCollection(),
              view = new PenaltyListView({collection: collection});

          application.main.show(view);
          collection.fetch();
        });
      });
    },

    addPenalty: function () {
      this._ifLogged(function () {
        require(['application', 'penalty/views/add', 'penalty/models/penalty'], function (application, PenaltyAddView, PenaltyModel) {
          var model = new PenaltyModel(),
              view = new PenaltyAddView({model: model});

          application.main.show(view);
        });
      });
    },

    viewPenalty: function (id) {
      this._ifLogged(function () {
        require(['application', 'penalty/views/view', 'penalty/models/penalty'], function (application, PenaltyViewView, PenaltyModel) {
          var model = new PenaltyModel({id: id}),
              view = new PenaltyViewView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    editPenalty: function (id) {
      this._ifLogged(function () {
        require(['application', 'penalty/views/edit', 'penalty/models/penalty'], function (application, PenaltyAddView, PenaltyModel) {
          var model = new PenaltyModel({id: id}),
              view = new PenaltyAddView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    // Shortcut to main routers _ifLogged method
    _ifLogged: function (callback) {
      G.routers.main._ifLogged.call(this, callback);
    }
  });

  return PenaltyRouter;
});
