define(
  ['marionette', 'hb!penalty/templates/edit', 'moment'],
  function (Marionette, penaltyEditTemplate, moment) {
  'use strict';

  var PenaltyEdit = Marionette.ItemView.extend({
    template: penaltyEditTemplate,

    events: {
      'submit form': 'savePenalty',
      'focus [type=date]': 'setDate'
    },

    modelEvents: {
      'change': 'render'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Punkty karne');
    },

    savePenalty: function (e) {
      var self = this;

      this.model.save({
        description: this.$el.find('[name=description]').val(),
        price: this.$el.find('[name=price]').val(),
        points: this.$el.find('[name=points]').val(),
        date_on: this.$el.find('[name=date_on]').val(),
        date_offense: this.$el.find('[name=date_offense]').val(),
        date_to: this.$el.find('[name=date_to]').val()
      }, {
        success: function (model, resp, options) {
          G.navigate('penalty');
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'description')
                .showErrorMessage(resp, 'price')
                .showErrorMessage(resp, 'points')
                .showErrorMessage(resp, 'date_on')
                .showErrorMessage(resp, 'date_offense')
                .showErrorMessage(resp, 'date_to')
              ;
              break;
          }
        }
      });

      return false;
    },

    setDate: function(e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + ']').next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return PenaltyEdit;
});
