define(
  ['marionette', 'module', 'hb!refuel/templates/edit', 'moment'],
  function (Marionette, module, refuelEditTemplate, moment) {
  'use strict';

  var RefuelEdit = Marionette.ItemView.extend({
    template: refuelEditTemplate,

    events: {
      'submit form': 'saveRefuel',
      'focus [type=date]': 'setDate'
    },

    modelEvents: {
      'change': 'render'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Tankowania');

      var i, self = this;
      $.get(module.config().fuelTypes, function (data, status, xhr) {
        self.model.set('fuelData', data);
      });
    },

    saveRefuel: function (e) {
      var self = this;

      this.model.save({
        date_prev_refuel: this.$el.find('[name=date_prev_refuel]').val(),
        course_before: this.$el.find('[name=course_before]').val(),
        date_refuel: this.$el.find('[name=date_refuel]').val(),
        course_after: this.$el.find('[name=course_after]').val(),
        fk_refuel_fuel_type_id: this.$el.find('[name=fk_refuel_fuel_type_id]').val(),
        liters: this.$el.find('[name=liters]').val(),
        price: this.$el.find('[name=price]').val(),
        description: this.$el.find('[name=description]').val()
      }, {
        success: function (model, resp, options) {
          G.navigate('refuel');
        },
        error: function (model, resp, options) {
          switch(resp.status) {
            case 400:
              self
                .showErrorMessage(resp, 'date_prev_refuel')
                .showErrorMessage(resp, 'course_before')
                .showErrorMessage(resp, 'date_refuel')
                .showErrorMessage(resp, 'course_after')
                .showErrorMessage(resp, 'fk_refuel_fuel_type_id')
                .showErrorMessage(resp, 'liters')
                .showErrorMessage(resp, 'price')
                .showErrorMessage(resp, 'description')
              ;
              break;
          }
        }
      });

      return false;
    },

    setDate: function(e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + ']').next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return RefuelEdit;
});
