define(['marionette'], function (Marionette) {
  'use strict';

  var RequestRouter = Marionette.AppRouter.extend({

    routes: {
      'request': 'requestList',
      'request/view/:id': 'requestView',
      'request/send/new/:user': 'requestNew',
      'request/send/message/:id(/:nothin)': 'requestReply',
      'request/send/reject/:id(/:nothin)': 'requestReject',
      'request/send/change/:id/:work': 'requestChange'
    },

    requestList: function () {
      this._ifLogged(function () {
        require(['application', 'request/views/list', 'request/collections/request'], function (application, RequestListView, RequestCollection) {
          var collection = new RequestCollection(),
              view = new RequestListView({collection: collection});

          application.main.show(view);
          collection.fetch();
        });
      });
    },

    requestView: function (id) {
      this._ifLogged(function () {
        require(['application', 'request/views/view', 'request/models/details'], function (application, RequestViewView, DetailsModel) {
          var model = new DetailsModel({id: id}),
              view = new RequestViewView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    requestNew: function (user) {
      this._ifLogged(function () {
        require(['application', 'request/views/send-new', 'request/models/lead'], function (application, RequestSendNewView, LeadModel) {
          var model = new LeadModel({user: user}),
              view = new RequestSendNewView({model: model});
          application.main.show(view);
        });
      });
    },

    requestReply: function (id) {
      this._ifLogged(function () {
        require(['application', 'request/views/send-reply', 'request/models/lead'], function (application, RequestSendReplyView, LeadModel) {
          var model = new LeadModel({id: id}),
              view = new RequestSendReplyView({model: model});
          application.main.show(view);
        });
      });
    },

    requestReject: function (id) {
      this._ifLogged(function () {
        require(['application', 'request/views/send-reject', 'request/models/lead'], function (application, RequestSendRejectView, LeadModel) {
          var model = new LeadModel({id: id}),
              view = new RequestSendRejectView({model: model});
          application.main.show(view);
        });
      });
    },

    requestChange: function (id, work) {
      this._ifLogged(function () {
        require(['application', 'request/views/send-change', 'request/models/lead'], function (application, RequestSendChangeView, LeadModel) {
          var model = new LeadModel({id: id, work: work}),
              view = new RequestSendChangeView({model: model});
          application.main.show(view);
        });
      });
    },

    // Shortcut to main routers _ifLogged method
    _ifLogged: function (callback) {
      G.routers.main._ifLogged.call(this, callback);
    }
  });

  return RequestRouter;
});
