define(
  ['marionette', 'module', 'hb!request/templates/send-new', 'moment'],
  function (Marionette, module, requestSendNewTemplate, moment) {
  'use strict';

  var RequestSendNew = Marionette.ItemView.extend({
    template: requestSendNewTemplate,

    events: {
      'submit form': 'sendNew',
      'focus [type=date]': 'setDate'
    },

    modelEvents: {
      'change': 'render'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Wiadomości');

      var self = this;
      $.get(module.config().workshops, function (data, status, xhr) {
        self.model.set('workshops', data);
      });

      _.bindAll(this, 'geolocationSuccess', 'geolocationError');
    },

    onRender: function () {
      this.getGeolocation();
    },

    getGeolocation: function () {
      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .in-progress').show();

      navigator.geolocation.getCurrentPosition(
        this.geolocationSuccess,
        this.geolocationError,
        { enableHighAccuracy: true }
      );
    },

    geolocationSuccess: function (location) {
      this.$el.find('[name=lat]').val(location.coords.latitude);
      this.$el.find('[name=lng]').val(location.coords.longitude);

      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .success').show();
    },

    geolocationError: function (error) {
      this.$el.find('[name=lat]').val('');
      this.$el.find('[name=lng]').val('');

      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .error').show();
    },

    sendNew: function () {
      var self = this;

      $.ajax({
        type: 'POST',
        url: module.config().send,
        data: JSON.stringify({
          request: {
            fk_user_user_id_to: this.model.get('user'),
            fk_workshop_type_id: this.$el.find('[name=fk_workshop_type_id]').val(),
            suggest_date: this.$el.find('[name=suggest_date]').val() + ' ' + this.$el.find('[name=suggest_time]').val(),
            lat: this.$el.find('[name=lat]').val(),
            lng: this.$el.find('[name=lng]').val(),
            message: this.$el.find('[name=message]').val()
          }
        }),
        dataType: 'json',

        success: function(data, status, xhr) {
          G.navigate('request');
        },

        error: function (xhr, status, error) {
          switch(xhr.status) {
            case 400:
              self
                .showErrorMessage(xhr, 'fk_workshop_type_id')
                .showErrorMessage(xhr, 'suggest_date')
                .showErrorMessage(xhr, 'message')
              ;
              break;
          }
        }
      });
      return false;
    },

    setDate: function (e) {
      var input = e.target,
          options = {mode: 'date'};

      input.blur();

      options.date = new Date(input.value);
      if (!(options.date instanceof Date) || isNaN(options.date.getTime())) {
        options.date = new Date();
      }

      datePicker.show(options, function(date) {
        if ((date instanceof Date) && !isNaN(date.getTime())) {
          input.value = moment(date).format('YYYY-MM-DD');
        }
      });

      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + '], .' + name).next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return RequestSendNew;
});
