define(
  ['marionette', 'module', 'hb!request/templates/send-reject'],
  function (Marionette, module, requestSendRejectTemplate) {
  'use strict';

  var RequestSendReject = Marionette.ItemView.extend({
    template: requestSendRejectTemplate,

    events: {
      'submit form': 'sendReject'
    },

    modelEvents: {
      'change': 'render'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Wiadomości');

      _.bindAll(this, 'geolocationSuccess', 'geolocationError');
    },

    onRender: function () {
      this.getGeolocation();
    },

    getGeolocation: function () {
      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .in-progress').show();

      navigator.geolocation.getCurrentPosition(
        this.geolocationSuccess,
        this.geolocationError,
        { enableHighAccuracy: true }
      );
    },

    geolocationSuccess: function (location) {
      this.$el.find('[name=lat]').val(location.coords.latitude);
      this.$el.find('[name=lng]').val(location.coords.longitude);

      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .success').show();
    },

    geolocationError: function (error) {
      this.$el.find('[name=lat]').val('');
      this.$el.find('[name=lng]').val('');

      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .error').show();
    },

    sendReject: function () {
      var self = this,
          id = this.model.get('id'),
          url = module.config().send + id;

      $.ajax({
        type: 'PUT',
        url: url,
        data: JSON.stringify({
          request: {
            lat: this.$el.find('[name=lat]').val(),
            lng: this.$el.find('[name=lng]').val(),
            message: this.$el.find('[name=message]').val()
          }
        }),
        dataType: 'json',

        success: function(data, status, xhr) {
          G.navigate('request/view/' + id);
        },

        error: function (xhr, status, error) {
          switch(xhr.status) {
            case 400:
              self
                .showErrorMessage(xhr, 'message')
              ;
              break;
          }
        }
      });
      return false;
    },

    showErrorMessage: function (resp, name) {
      var label = this.$el.find('[name=' + name + '], .' + name).next('.error');
      if (resp.responseJSON[name]) {
        label.text(resp.responseJSON[name]);
      } else {
        label.text('');
      }
      return this;
    }
  });

  return RequestSendReject;
});
