define(['marionette'], function (Marionette) {
  'use strict';

  var SearchRouter = Marionette.AppRouter.extend({

    routes: {
      'search': 'searchForm',
      'search/result/:query': 'searchResult',
      'search/result-map/:query': 'searchResultMap',
      'search/details/:id': 'searchDetails',
      'search/local-list': 'localList',
      'search/local-details/:id': 'localDetails'
    },

    searchForm: function () {
      this._ifLoggedOrOffline(function () {
        require(['application', 'search/views/form'], function (application, SearchFormView) {
          var view = new SearchFormView();
          application.main.show(view);
        });
      });
    },

    searchResult: function (query) {
      this._ifLogged(function () {
        require(['application', 'search/views/result', 'search/collections/result'], function (application, SearchResultView, ResultCollection) {
          var collection = new ResultCollection(),
              view = new SearchResultView({collection: collection});

          collection.query = query;
          application.main.show(view);
          collection.fetch();
        });
      });
    },

    searchResultMap: function (query) {
      this._ifLogged(function () {
        require(['application', 'search/views/result-map', 'search/collections/result'], function (application, SearchResultMapView, ResultCollection) {
          var collection = new ResultCollection(),
              view = new SearchResultMapView({collection: collection});

          collection.query = query;
          application.main.show(view);
          collection.fetch();
        });
      });
    },

    searchDetails: function (id) {
      this._ifLogged(function () {
        require(['application', 'search/views/details', 'search/models/details'], function (application, SearchDetailsView, DetailsModel) {
          var model = new DetailsModel({id: id}),
              view = new SearchDetailsView({model: model});

          application.main.show(view);
          model.fetch();
        });
      });
    },

    localList: function () {
      this._ifLoggedOrOffline(function () {
        require(['application', 'search/views/offline', 'search/collections/result'], function (application, SearchOfflineView, ResultCollection) {
          var collectionData = [],
              collection = new ResultCollection(),
              view = new SearchOfflineView({collection: collection});

          try {
            collectionData = JSON.parse(localStorage['search/local-list/data']);
          } catch (e) {}

          if (collectionData instanceof Array) {
            collection.set(collectionData);
          }

          application.main.show(view);
        });
      });
    },

    localDetails: function (id) {
      this._ifLoggedOrOffline(function () {
        require(['application', 'search/views/offline-details', 'search/models/details'], function (application, SearchOfflineDetailsView, DetailsModel) {
          var collectionData = [], i,
              model = new DetailsModel(),
              view = new SearchOfflineDetailsView({model: model});

          try {
            collectionData = JSON.parse(localStorage['search/local-list/data']);
          } catch (e) {}

          if (collectionData instanceof Array) {
            for (i=0; i< collectionData.length; i++) {
              if (id == collectionData[i].id) {
                model.set(collectionData[i]);
              }
            }
          }

          application.main.show(view);
        });
      });
    },

    // Shortcut to main routers _ifLogged method
    _ifLogged: function (callback) {
      G.routers.main._ifLogged.call(this, callback);
    },

    // Shortcut to main routers _ifLoggedOrOffline method
    _ifLoggedOrOffline: function (callback) {
      G.routers.main._ifLoggedOrOffline.call(this, callback);
    }
  });

  return SearchRouter;
});
