define(['underscore', 'marionette', 'hb!search/templates/form'], function (_, Marionette, formTemplate) {
  'use strict';

  var FormView = Marionette.ItemView.extend({
    template: formTemplate,

    events: {
      'submit form': 'search',
      'click .location-status .error': 'getGeolocation'
    },

    initialize: function () {
      G.vent.trigger('header-title:change', 'Wyszukiwarka');

      _.bindAll(this, 'geolocationSuccess', 'geolocationError');
    },

    onRender: function () {
      this.getGeolocation();
    },

    getGeolocation: function () {
      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .in-progress').show();

      navigator.geolocation.getCurrentPosition(
        this.geolocationSuccess,
        this.geolocationError,
        { enableHighAccuracy: true }
      );
    },

    geolocationSuccess: function (location) {
      this.$el.find('[name=lat]').val(location.coords.latitude);
      this.$el.find('[name=lng]').val(location.coords.longitude);

      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .success').show();
    },

    geolocationError: function (error) {
      this.$el.find('[name=lat]').val('');
      this.$el.find('[name=lng]').val('');

      this.$el.find('.location-status span').hide();
      this.$el.find('.location-status .error').show();
    },

    search: function () {
      G.navigate('search/result/' + this.$el.find('form').serialize());
      return false;
    }
  });

  return FormView;
});
